Immediate

Struct Immediate 

pub struct Immediate;
Immediate execution strategy.

This strategy executes a given task immediately, and is primarily intended for testing and debugging purposes. No threading is involved, which makes it much easier to reason about execution order. Additionally, it can act as a baseline for comparison with worker-based strategies.

Warning: When channels are involved, this strategy might deadlock, i.e., when a task from a worker thread tries to send on a bounded channel that is full. In this case, make sure to either use unbounded channels, or actively drain the channel before sending on it.

Examples

use zrx_executor::strategy::{Immediate, Strategy};

// Create strategy and submit task
let strategy = Immediate::default();
strategy.submit(Box::new(|| println!("Task")))?;

Implementations

Source

impl Immediate

pub fn new() -> Self

Creates an immediate execution strategy.

Examples
use zrx_executor::strategy::Immediate;

// Create strategy
let strategy = Immediate::new();