WorkSharing

Struct WorkSharing 

pub struct WorkSharing { /* private fields */ }
Work-sharing execution strategy.

This strategy manages its tasks centrally in a single bounded crossbeam channel, which pull tasks from it and execute them, repeating the process until they are terminated. This is a very simple, yet reasonably efficient strategy in most cases.

Tasks are processed in the exact same order they were submitted, albeit they might not finish in the same order. As this strategy uses a bounded channel, task submission might fail when the channel’s capacity is reached, leading to better performance characteristics due to the use of atomics over locks. As an alternative, the WorkStealing strategy can be used, which is built on unbounded channels and allows for more flexible task submission, including automatic load balancing between workers which is particularly useful when tasks create subtasks.

Examples

use zrx_executor::strategy::{Strategy, WorkSharing};

// Create strategy and submit task
let strategy = WorkSharing::default();
strategy.submit(Box::new(|| println!("Task")))?;

Implementations

Source

impl WorkSharing

pub fn new(num_workers: usize) -> Self

Creates a work-sharing execution strategy.

This method creates a strategy with the given number of worker threads, which are spawned immediately before the method returns. Internally, a bounded channel is created with a capacity of 8 tasks per worker, so for 4 workers, the channel will have a capacity of 32 tasks.

Use WorkSharing::with_capacity to set a custom capacity.

Panics

Panics if thread creation fails.

Examples
use zrx_executor::strategy::WorkSharing;

// Create strategy
let strategy = WorkSharing::new(4);
pub fn with_capacity(num_workers: usize, capacity: usize) -> Self

Creates a work-sharing execution strategy with the given capacity.

This method creates a strategy with the given number of worker threads, which are spawned immediately before the method returns.

This strategy makes use of a bounded channel for its better performance characteristics, since the caller is expected to have control over task submission, ensuring that the executor can accept new tasks. The given capacity sets the number of tasks the executor accepts before starting to reject them, which can be used to apply backpressure. Note that the capacity is not a per-worker, but a global per-executor limit.

Panics

Panics if thread creation fails.

Examples
use zrx_executor::strategy::WorkSharing;

// Create strategy with capacity
let strategy = WorkSharing::with_capacity(4, 64);