WorkStealing

Struct WorkStealing 

pub struct WorkStealing { /* private fields */ }
Work-stealing execution strategy.

This strategy implements work-stealing, where each worker thread has its own local queue. Workers can steal tasks from a central injector or from other workers, if their local queues are empty. This allows for more efficient execution if there’s a large number of workers and tasks.

Work stealing enhances load balancing by allowing idle workers to take on tasks from busier peers, which helps to reduce idle time and can improve overall throughput. Unlike the simpler WorkSharing strategy that uses a central queue where workers may become stuck waiting for new tasks, this method can yield better resource utilization. Additionally, work-stealing helps mitigate contention over shared resources (i.e. channels), which can become a bottleneck in central queueing systems, allowing each worker to primarily operate on its local queue. However, if task runtimes are short, the utilization can be lower than with a central queue, since stealing uses an optimistic strategy and is a best-effort operation. When task runtimes are long, work stealing can be more efficient than central queueing.

This reduced contention is particularly beneficial in dynamic environments with significantly fluctuating workloads, enabling faster task completion as workers can quickly adapt to take on shorter or less complex tasks as they become available.

Examples

use zrx_executor::strategy::{Strategy, WorkStealing};

// Create strategy and submit task
let strategy = WorkStealing::default();
strategy.submit(Box::new(|| println!("Task")))?;

Implementations

Source

impl WorkStealing

pub fn new(num_workers: usize) -> Self

Creates a work-stealing execution strategy.

This method creates a strategy with the given number of worker threads, which are spawned immediately before the method returns. Internally, a default limit of 8 tasks per worker is used, so for 4 workers, the executor will have a capacity of 32 tasks.

Use WorkStealing::with_capacity to set a custom capacity.

Panics

Panics if thread creation fails.

Examples
use zrx_executor::strategy::WorkStealing;

// Create strategy
let strategy = WorkStealing::new(4);
pub fn with_capacity(num_workers: usize, capacity: usize) -> Self

Creates a work-stealing execution strategy with the given capacity.

This method creates a strategy with the given number of worker threads, which are spawned immediately before the method returns.

While this strategy uses unbounded channels due to how the Injector and Worker concepts are implemented, it still provides a capacity limit to ensure the executor doesn’t end being overwhelmed. The given capacity sets the number of tasks the executor accepts before starting to reject them, which can be used to apply backpressure. Note that the capacity is not a per-worker, but a global per-executor limit.

Panics

Panics if thread creation fails.

Examples
use zrx_executor::strategy::WorkStealing;

// Create strategy
let strategy = WorkStealing::with_capacity(4, 64);