Tasks

Struct Tasks 

pub struct Tasks { /* private fields */ }
Task set.

This data type represents a set of tasks that can either be consumed through iteration, or executed recursively via Tasks::execute. Anything returned by Task::execute must be convertible into Tasks, including a single task, multiple tasks, and the unit value.

Examples

use zrx_executor::task::Tasks;
use zrx_executor::Executor;

// Create executor and submit task
let executor = Executor::default();
executor.submit(|| {
    println!("Task 1");

    // Create subtasks
    let mut tasks = Tasks::new();
    tasks.add(|| println!("Task 1.1"));
    tasks.add(|| println!("Task 1.2"));
    tasks.add(|| println!("Task 1.3"));
    tasks
})?;

Implementations

Source

impl Tasks

pub fn new() -> Self

Creates a task set.

Examples
use zrx_executor::task::Tasks;

// Create task set
let tasks = Tasks::new();
pub fn add<T>(&mut self, task: T) -> &mut Self
where T: Task,

Adds a task to the task set.

This method adds a Task to the set, which can either be consumed via Tasks::into_iter or executed via Tasks::execute.

Examples
use zrx_executor::task::Tasks;

// Create task set and add tasks
let mut tasks = Tasks::new();
tasks.add(|| println!("Task 1"));
tasks.add(|| println!("Task 2"));
tasks.add(|| println!("Task 3"));
pub fn execute(self)

Executes all tasks in the task set.

This method executes all tasks recursively in depth-first order, so if a task returns further subtasks, they are executed before all others.

Examples
use zrx_executor::task::Tasks;

// Create task set and add tasks
let mut tasks = Tasks::new();
tasks.add(|| println!("Task 1"));
tasks.add(|| println!("Task 2"));
tasks.add(|| println!("Task 3"));

// Execute task set
tasks.execute();
Source

impl Tasks

pub fn len(&self) -> usize

Returns the number of tasks.

pub fn is_empty(&self) -> bool

Returns whether there are any tasks.